<?php

namespace Wicked_Block_Builder;

// Disable direct load
if ( ! defined( 'ABSPATH' ) ) {
	die( 'Access denied.' );
}

/**
 * Holds a collection of blocks.
 */
class Block_Collection extends Object_Collection implements \JsonSerializable {

    /**
     * Add a block.
     *
     * @param Block
     *  The block to add.
     */
    public function add( $item ) {
        $this->add_if( $item, 'Wicked_Block_Builder\Block' );
    }

	public function jsonSerialize() {
		return $this->items;
	}

	/**
	 * Populates the collection with the current version of published blocks and
	 * returns the collection.
	 *
	 * @return Block_Collection
	 */
	public function get_published_blocks() {
		// Get published blocks
		$blocks = get_posts( array(
		    'post_type'         => Block::$post_type_name,
		    'posts_per_page'    => -1,
			'post_status' 		=> 'publish',
		) );

		foreach ( $blocks as $post ) {
			$block = new Block();
			$block->from_post( $post );

			$this->add( $block );
		}

		return $this;
	}

	/**
	 * Returns the CSS for the blocks in the collection.
	 *
	 * @return string
	 */
	public function get_css() {
		$css = array();

		foreach ( $this->items as $block ) {
			$css[] = $block->css;
		}

		$css = implode( "\n\n", $css );
		$css .= "\n";

		return $css;
	}

	/**
	 * Registers all blocks in the collection.
	 */
	public function register() {
		foreach ( $this->items as $block ) {
			$block->register();
		}

		return $this;
	}
}
